import logging

from astropy import wcs
from astropy.io import fits
from astropy.visualization import ZScaleInterval
from astropy.utils.decorators import lazyproperty

import pyqtgraph as pg
from pyqtgraph.Qt import QtWidgets
from pgcolorbar.colorlegend import ColorLegendItem

from .ViewerElement import ViewerElement
from ..runtime import RuntimeData


logger = logging.getLogger(__name__)


class ImageCutout(ViewerElement):
    def __init__(self, rd: RuntimeData, parent=None):
        self.cfg = rd.config.viewer.image_cutout
        super().__init__(rd=rd, cfg=self.cfg, parent=parent)

        # TODO: introduce a class attribute `grid` and move it to ViewerElement?
        grid = QtWidgets.QGridLayout()
        grid.setContentsMargins(0, 0, 0, 0)

        # add a label
        self._label = QtWidgets.QLabel()
        grid.addWidget(self._label, 1, 1)

        # add a widget for the image
        self._image_widget = pg.GraphicsLayoutWidget()
        grid.addWidget(self._image_widget, 2, 1)

        self.setLayout(grid)

        # set up the color map
        self._cmap = pg.colormap.get('viridis')

        # set up the image and the view box
        self._image = pg.ImageItem(border='k')
        self._image.setLookupTable(self._cmap.getLookupTable())
        self._view_box = self._image_widget.addViewBox(0, 0)
        self._view_box.addItem(self._image)
        self._view_box.setAspectLocked(True)

        # set up the color bar
        self._cbar = ColorLegendItem(imageItem=self._image, showHistogram=True)
        self._image_widget.addItem(self._cbar, 0, 1)

    @lazyproperty
    def _data(self):
        try:
            data = fits.getdata(self._filename)
            data = data * 1e21
        except ValueError:
            logger.warning('Image cutout not found (object ID: {})'.format(self.rd.id))
            return
        else:
            return data

    def reset_view(self):
        if self._data is None:
            return

        self._cbar.setLevels(ZScaleInterval().get_limits(self._data))
        self._view_box.autoRange()

    def load_object(self):
        super().load_object()

        del self._data

        if self._data is not None:
            self.setEnabled(True)

            self._label.setText("Image: {}".format(self._filename.name))
            self._image.setImage(self._data)

            self.reset_view()
        else:
            self._label.setText("")
            self._image.clear()
            self.setEnabled(False)

    @staticmethod
    def radec_to_pix(ra_coords, dec_coords, header, origin=0):
        """
        Converts RA & DEC world coordinates to pixel coordinates.

        In:
        ---
        ra_coords ... 1D array of RA in degrees (float)
        dec_coords ... 1D array of corresponding DEC in degrees (float)
        header ... an astropy.io.fits header object
        origin ... output coordinates 0-indexed if 0, or 1-indexed if 1
                   (default: 0)

        Out:
        ---
        x_coords, y_coords ... 2-tuple of 1D arrays with pixel coordinates
        """

        wcs_obj = wcs.WCS(header, relax=False)  # no HST only spec
        # allowed, only WCS
        # standard
        coords = wcs_obj.wcs_world2pix(ra_coords, dec_coords, origin)
        x_coords = coords[0]
        y_coords = coords[1]
        return x_coords, y_coords

    def plot(self):
        # width_window_pix = width_window[0] / band_ps
        # self.view_cutout.setRange(QtCore.QRectF(x_band-width_window_pix/2.,
        #                                     y_band-width_window_pix/2.,
        #                                     width_window_pix,
        #                                     width_window_pix))


        ### TODO: position of lines seems shifted and doesn't match with catalogue

        # test_line = pg.InfiniteLine(angle=90,movable=False,
        #                            pen=pg.mkPen(color=c_data_crossline, width = 1))
        # test_line.setPos([x_band,0])
        # test_line2 = pg.InfiniteLine(angle=0,movable=False,
        #                             pen=pg.mkPen(color=c_data_crossline, width = 1))
        # test_line2.setPos([0,y_band])
        # self.view_cutout.addItem(test_line)
        # self.view_cutout.addItem(test_line2)
        return



from pyqtgraph.Qt import QtWidgets

from ..runtime import RuntimeData
from .AbstractWidget import AbstractWidget


class ReviewForm(QtWidgets.QGroupBox, AbstractWidget):
    def __init__(self, rd: RuntimeData, parent=None):
        self.cfg = rd.config.review_form
        super().__init__(rd=rd, cfg=self.cfg, parent=parent)

        self.setTitle('Review Form')
        self.setSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)

        grid = QtWidgets.QGridLayout()

        # add checkboxes
        self._checkboxes = {}

        if self.cfg.checkboxes:
            for i, (cname, label) in enumerate(self.cfg.checkboxes.items()):
                widget = QtWidgets.QCheckBox(label)
                self._checkboxes[cname] = widget
                grid.addWidget(widget, i + 1, 1, 1, 1)

        # add a multi-line text editor for writing comments
        self._comments_widget = QtWidgets.QTextEdit()
        grid.addWidget(QtWidgets.QLabel('Comments:'), len(self._checkboxes) + 1, 1, 1, 1)
        grid.addWidget(self._comments_widget, len(self._checkboxes) + 2, 1, 1, 1)

        self.setLayout(grid)

    def dump(self):
        self.rd.df.at[self.rd.id, 'comment'] = self._comments_widget.toPlainText()
        for cname, widget in self._checkboxes.items():
            self.rd.df.at[self.rd.id, cname] = widget.isChecked()

    def load_object(self):
        self._comments_widget.setText(self.rd.df.at[self.rd.id, 'comment'])
        for cname, widget in self._checkboxes.items():
            widget.setChecked(self.rd.df.at[self.rd.id, cname])



import logging
import pathlib

import numpy as np
from astropy.io import fits
from astropy.table import Table


logger = logging.getLogger(__name__)


# TODO: create FilenameParser class

def get_grizli_id(filename: pathlib.Path) -> str:
    """
    Retrieve a grizli ID from the file name. The file can be any of the grizli fit products.
    @param filename: `~pathlib.Path`
        The file name to parse.
    @return: str
        The object ID.
    """
    return filename.name.split('_')[1].split('.')[0]


def load_cat(filename=None, translate=None, data_folder=None, filename_parser=get_grizli_id) -> [Table, None]:
    """
    Read and filter the input catalogue.

    @return:
        The processed catalogue.
    """

    if filename is None:
        logger.warning('Catalogue filename not specified')
        return

    # read the input catalogue
    try:
        cat = Table(fits.getdata(filename))
    except OSError:
        logger.error('Could not load the catalogue')
        return

    # rename columns
    if translate is not None:
        for cname, cname_synonyms in translate.items():
            for syn in cname_synonyms:
                if syn in cat.colnames:
                    cat.rename_column(syn, cname)
                    break

    if 'id' not in cat.colnames:
        logger.error(column_not_found_message('id', translate))
        return
    cat.add_index('id')

    # scan the data folder
    if data_folder is not None:
        data_files = sorted(pathlib.Path(data_folder).glob('**/*.fits'))

        try:
            obj_ids = np.unique([filename_parser(p) for p in data_files])
        except IndexError:
            logger.error('An error occurred when parsing filenames in the data folder')
            return

        if not obj_ids.size:
            logger.error('No IDs retrieved from the data folder')
            return

        # filter objects using a list of IDs retrieved from the data folder
        cat = cat[np.in1d(cat['id'], obj_ids, assume_unique=True)]

    if len(cat) == 0:
        logger.error('The processed catalogue is empty')
        return

    return cat


def column_not_found_message(cname, translate=None):
    if translate is None or cname not in translate:
        return '`{}` column not found'.format(cname)
    else:
        return '`{}` column and its equivalences ({}) not found'.format(cname, ", ".join(translate[cname]))


def get_filename(directory, search_mask, object_id):
    matched_filenames = sorted(list(pathlib.Path(directory).glob(search_mask.format(object_id))))
    if matched_filenames:
        return matched_filenames[0]  # if more than one filename is matched to `search_mask`, return the first
    else:
        return



import logging

from astropy.coordinates import SkyCoord

from ..runtime import RuntimeData
from .AbstractWidget import AbstractWidget


from pyqtgraph.Qt import QtWidgets, QtCore


logger = logging.getLogger(__name__)


class ObjectInfo(QtWidgets.QGroupBox, AbstractWidget):
    def __init__(self, rd: RuntimeData, parent=None):
        self.cfg = rd.config.object_info
        super().__init__(rd=rd, cfg=self.cfg, parent=parent)

        self.setTitle('Object Information')
        self.setSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)

        grid = QtWidgets.QGridLayout()

        # display information about the object
        self._labels = {}

        if self.cfg.items:
            for i, (cname, label) in enumerate(self.cfg.items.items()):
                widget = QtWidgets.QLabel()
                widget.setText(label)
                widget.setHidden(True)
                widget.setTextInteractionFlags(QtCore.Qt.TextSelectableByMouse)
                self._labels[cname] = widget
                grid.addWidget(widget, i + 1, 1, 1, 1)

        self.setLayout(grid)

    def load_object(self):
        for cname, widget in self._labels.items():
            try:
                widget.setText(self.cfg.items[cname].format(self.rd.cat.loc[self.rd.id][cname]))
            except KeyError:
                if cname in self.rd.cat.colnames:
                    logger.warning('Object not found in the catalogue (ID: {})'.format(self.rd.id))
                else:
                    logger.warning('`{}` column not found in the catalogue'.format(cname))
                widget.setHidden(True)
            else:
                widget.setHidden(False)

        # if 'ra' in self._cat.colnames and 'dec' in self._cat.colnames:
        #     c = SkyCoord(ra=self._cat['ra'][self._j], dec=self._cat['dec'][self._j], frame='icrs', unit='deg')
        #     ra, dec = c.to_string('hmsdms').split(' ')
        #     self.ra_label.setText("RA: {}".format(ra))
        #     self.dec_label.setText("Dec: {}".format(dec))



from pyqtgraph.Qt import QtWidgets

from ..runtime import RuntimeData
from ..utils import params


class AbstractWidget(QtWidgets.QWidget):
    def __init__(self, rd: RuntimeData, cfg: params.AbstractWidget, parent=None):
        self.rd = rd

        super().__init__(parent)

        if cfg.min_width:
            self.setMinimumWidth(cfg.min_width)
        if cfg.min_height:
            self.setMinimumHeight(cfg.min_height)

        self.setEnabled(False)

    def dump(self):
        pass

    def load_object(self):
        pass

    def load_project(self):
        self.setEnabled(True)



from pyqtgraph.Qt import QtWidgets

from .AbstractWidget import AbstractWidget
from .ViewerElement import ViewerElement
from .ImageCutout import ImageCutout
from .Spec2D import Spec2D
from .Spec1D import Spec1D

from ..runtime import RuntimeData
from ..utils.widgets import get_widgets


class DataViewer(AbstractWidget):
    def __init__(self, rd: RuntimeData, parent=None):
        super().__init__(rd=rd, cfg=rd.config.viewer, parent=parent)

        grid = QtWidgets.QGridLayout()
        grid.setSpacing(10)
        grid.setContentsMargins(0, 0, 0, 0)

        # add a widget for the image cutout
        self.image_cutout = ImageCutout(self.rd, parent=self)
        grid.addWidget(self.image_cutout, 1, 1, 1, 1)

        # add a widget for the 2D spectrum
        self.spec_2D = Spec2D(self.rd, parent=self)
        grid.addWidget(self.spec_2D, 2, 1, 1, 1)

        # add a widget for the 1D spectrum
        self.spec_1D = Spec1D(self.rd, parent=self)
        grid.addWidget(self.spec_1D, 3, 1, 1, 1)

        self.setLayout(grid)

        self.widgets: list[ViewerElement] = []
        for w in get_widgets(grid):
            self.widgets.append(w)

    def load_object(self):
        for w in self.widgets:
            w.load_object()

    def reset_view(self):
        for w in self.widgets:
            w.reset_view()



import pathlib
import argparse
import sys
import logging
from importlib.metadata import version

import pyqtgraph as pg
from pyqtgraph.Qt import QtGui, QtWidgets

from .runtime import RuntimeData
from .menu import NewFile
from .widgets import (AbstractWidget, DataViewer, ControlPanel, ObjectInfo, ReviewForm)
from .utils.widgets import get_widgets
from .utils.logs import LogMessageBox

pg.setConfigOption('background', 'w')
pg.setConfigOption('foreground', 'k')
pg.setConfigOption('antialias', True)


logger = logging.getLogger(__name__)


class MainWindow(QtWidgets.QMainWindow):
    def __init__(self, parent=None):
        super().__init__(parent)
        logger.info("Application started")

        self.rd = RuntimeData()

        # size, title and logo
        self.setGeometry(600, 500, 2550, 1450)  # position and size of the window
        self.setWindowTitle('Specvizitor')  # title of the window
        self.setWindowIcon(QtGui.QIcon('logo2_2.png'))  # logo in upper left corner

        # add a menu
        self._add_menu()

        # add a status bar
        # self.statusBar().showMessage("Message in statusbar.")

        # add a central widget
        self.main_GUI = FRESCO(self.rd, parent=self)
        # self.main_GUI.signal1.connect(self.show_status)
        self.setCentralWidget(self.main_GUI)

        if self.rd.cache.last_inspection_file:
            self.open(self.rd.cache.last_inspection_file)

    def _add_menu(self):
        self._menu = self.menuBar()

        self._file = self._menu.addMenu("&File")

        self._new_file = QtWidgets.QAction("&New...")
        self._new_file.triggered.connect(self._new_file_action)
        self._file.addAction(self._new_file)

        self._open_file = QtWidgets.QAction("&Open...")
        self._open_file.triggered.connect(self._open_file_action)
        self._file.addAction(self._open_file)

        self._file.addSeparator()

        self._save = QtWidgets.QAction("&Save...")
        self._save.triggered.connect(self._save_action)
        self._save.setEnabled(False)
        self._file.addAction(self._save)

        self.shortcut_close = QtWidgets.QShortcut(QtGui.QKeySequence('Ctrl+S'), self)
        self.shortcut_close.activated.connect(self._save_action)

        self._save_as = QtWidgets.QAction("Save As...")
        self._save_as.triggered.connect(self._save_as_action)
        self._save_as.setEnabled(False)
        self._file.addAction(self._save_as)

        self._export = QtWidgets.QAction("&Export...")
        self._export.triggered.connect(self._export_action)
        self._export.setEnabled(False)
        self._file.addAction(self._export)

        self._file.addSeparator()

        self._exit = QtWidgets.QAction("E&xit...")
        self._exit.triggered.connect(self._exit_action)
        self._file.addAction(self._exit)

        self._tools = self._menu.addMenu("&Tools")
        self._settings = QtWidgets.QAction("Se&ttings...")
        self._settings.triggered.connect(self._settings_action)
        self._tools.addAction(self._settings)

        self._help = self._menu.addMenu("&Help")
        self._about = QtWidgets.QAction("&About...")
        self._about.triggered.connect(self._about_action)
        self._help.addAction(self._about)

    def _new_file_action(self):
        dialog = NewFile(self.rd, parent=self)
        if dialog.exec():
            self.rd.cache.last_object_index = 0
            self.load_project()

    def _open_file_action(self):
        path = QtWidgets.QFileDialog.getOpenFileName(self, caption='Open Inspection File', filter='CSV Files (*.csv)')[
            0]
        if path:
            self.rd.cache.last_object_index = 0
            self.open(path)

    def open(self, path: str):
        if pathlib.Path(path).exists():
            self.rd.output_path = pathlib.Path(path)
            self.rd.read()
            self.load_project()
        else:
            logger.warning('Inspection file not found (path: {})'.format(path))

    def load_project(self):
        for w in (self._save, self._save_as, self._export):
            w.setEnabled(True)
        self.setWindowTitle('{} – Specvizitor'.format(self.rd.output_path.name))
        self.main_GUI.load_project()

    def _save_action(self):
        msg = 'The data is saved automatically'
        if self.rd.output_path is not None:
            msg += ' to {}'.format(self.rd.output_path)
        LogMessageBox(logging.INFO, msg, parent=self)

    def _save_as_action(self):
        LogMessageBox(logging.INFO, 'Not implemented', parent=self)

    def _export_action(self):
        LogMessageBox(logging.INFO, 'Not implemented', parent=self)

    def _exit_action(self):
        self.rd.save()
        self.close()
        logger.info("Application closed")

    def _settings_action(self):
        QtWidgets.QMessageBox.information(self, "Settings",
                                          "Configuration file: {}\n\nCache: {}".
                                          format(self.rd.config_file.path, self.rd.cache_file.path))

    def _about_action(self):
        QtWidgets.QMessageBox.about(self, "About Specvizitor", "Specvizitor v{}".format(version('specvizitor')))

    def closeEvent(self, _):
        self._exit_action()


class FRESCO(QtWidgets.QWidget):
    def __init__(self, rd: RuntimeData, parent=None):
        self.rd = rd
        super().__init__(parent)

        # set up the widget layout
        self.layout = QtWidgets.QGridLayout()
        self.layout.setSpacing(10)
        self.setLayout(self.layout)

        # add a widget for the data viewer
        self.data_viewer = DataViewer(self.rd, parent=self)
        self.layout.addWidget(self.data_viewer, 1, 1, 3, 1)

        # add a widget for the control panel
        self.control_panel = ControlPanel(self.rd, parent=self)
        self.layout.addWidget(self.control_panel, 1, 2, 1, 1)

        # add a widget for displaying information about the object
        self.object_info = ObjectInfo(self.rd, parent=self)
        self.layout.addWidget(self.object_info, 2, 2, 1, 1)

        # add a widget for writing comments
        self.review_form = ReviewForm(self.rd, parent=self)
        self.layout.addWidget(self.review_form, 3, 2, 1, 1)

        # add the Eazy widget
        # self.eazy = Eazy(self)
        # grid.addWidget(self.eazy, 6, 3, 1, 1)

        # Write eazy results

        '''
        z_raw_chi2 = np.round(self.zout['z_raw_chi2'][self.j],3)
        eazy_raw_chi2 = QtWidgets.QLabel('Chi2: '+str(z_raw_chi2), self)
        grid.addWidget(eazy_raw_chi2,7,31,1,1)

        raw_chi2 = np.round(self.zout['raw_chi2'][self.j],3)
        eazy_raw_chi2 = QtWidgets.QLabel('Chi2: '+str(raw_chi2), self)
        grid.addWidget(eazy_raw_chi2,8,31,1,1)
        '''

        # self.z_phot_chi2 = np.round(self.zout['z_phot_chi2'][self.j], 3)
        # eazy_raw_chi2 = QtWidgets.QLabel('Chi2: ' + str(self.z_phot_chi2), self)
        # grid.addWidget(eazy_raw_chi2, 9, 31, 1, 1)
        #
        # self.sfr = np.round(self.zout['sfr'][self.j], 3)
        # eazy_sfr = QtWidgets.QLabel('SFR: ' + str(self.sfr), self)
        # grid.addWidget(eazy_sfr, 10, 31, 1, 1)
        #
        # self.mass = np.round(self.zout['mass'][self.j] / 10 ** 9, 3)
        # eazy_mass = QtWidgets.QLabel('mass: ' + str(self.mass), self)
        # grid.addWidget(eazy_mass, 11, 31, 1, 1)

        # self.control_panel.reset_button_clicked.connect(self.image_cutout.reset_view)

        self.control_panel.object_selected.connect(self.load_object)
        self.control_panel.reset_button_clicked.connect(self.data_viewer.reset_view)

    @property
    def widgets(self) -> list[AbstractWidget]:
        return get_widgets(self.layout)

    def load_object(self, j: int):
        if self.rd.j is not None:
            for widget in self.widgets:
                widget.dump()
            self.rd.save()

        self.rd.j = j
        self.rd.cache.last_object_index = j
        self.rd.cache.save(self.rd.cache_file)

        for widget in self.widgets:
            widget.load_object()

    def load_project(self):
        self.rd.cache.last_inspection_file = str(self.rd.output_path)
        self.rd.cache.save(self.rd.cache_file)

        # reload the review form
        self.layout.removeWidget(self.review_form)
        self.review_form.setParent(None)
        self.review_form = ReviewForm(self.rd, parent=self)
        self.layout.addWidget(self.review_form, 3, 2, 1, 1)

        for w in self.widgets:
            w.load_project()

        j = self.rd.cache.last_object_index

        if j and j < self.rd.n_objects:
            self.load_object(int(j))
        else:
            self.load_object(0)


def main():
    parser = argparse.ArgumentParser()

    # logging configuration
    parser.add_argument('-v', '--verbose', action='store_true')
    args = parser.parse_args()

    if args.verbose:
        level = logging.DEBUG
    else:
        level = logging.WARNING
    logging.basicConfig(format='%(asctime)s - %(name)s - %(levelname)s - %(message)s', level=level)

    app = QtWidgets.QApplication(sys.argv)
    window = MainWindow()
    window.show()
    sys.exit(app.exec_())


if __name__ == '__main__':
    main()



import logging

import numpy as np
import pyqtgraph as pg
from astropy.io import fits
from astropy.visualization import ZScaleInterval
from astropy.utils.decorators import lazyproperty

from pyqtgraph.Qt import QtWidgets
from pgcolorbar.colorlegend import ColorLegendItem

from .ViewerElement import ViewerElement
from ..runtime import RuntimeData


logger = logging.getLogger(__name__)


class Spec2D(ViewerElement):
    def __init__(self, rd: RuntimeData, parent=None):
        self.cfg = rd.config.viewer.spec_2d
        super().__init__(rd=rd, cfg=self.cfg, parent=parent)

        grid = QtWidgets.QGridLayout()
        grid.setContentsMargins(0, 0, 0, 0)

        # add a label
        self._label = QtWidgets.QLabel()
        grid.addWidget(self._label, 1, 1)

        # add a widget for the spectrum
        self._spec_2d_widget = pg.GraphicsLayoutWidget()
        grid.addWidget(self._spec_2d_widget, 2, 1)

        self.setLayout(grid)

        # set up the color map
        self._cmap = pg.colormap.get('viridis')

        # set up the image and the view box
        self._spec_2d = pg.ImageItem(border='k')
        self._spec_2d.setLookupTable(self._cmap.getLookupTable())
        self._view_box = self._spec_2d_widget.addViewBox(0, 0)
        self._view_box.addItem(self._spec_2d)
        self._view_box.setAspectLocked(True)

        # set up the color bar
        self._cbar = ColorLegendItem(imageItem=self._spec_2d, showHistogram=True, histHeightPercentile=99.0)
        self._spec_2d_widget.addItem(self._cbar, 0, 1)

    @lazyproperty
    def _data(self):
        try:
            data = fits.getdata(self._filename)
            data = np.rot90(data)[::-1]
        except ValueError:
            logger.warning('2D spectrum not found (object ID: {})'.format(self.rd.id))
            return
        else:
            return data

    def reset_view(self):
        if self._data is None:
            return

        # TODO: allow to choose between min/max and zscale?
        self._cbar.setLevels(ZScaleInterval().get_limits(self._data))
        self._view_box.autoRange()

    def load_object(self):
        super().load_object()

        del self._data

        if self._data is not None:
            self.setEnabled(True)

            self._label.setText("2D spectrum: {}".format(self._filename.name))
            self._spec_2d.setImage(self._data)

            self.reset_view()
        else:
            self._label.setText("")
            self._spec_2d.clear()

            self.setEnabled(False)



import logging
from functools import wraps

from pyqtgraph.Qt import QtWidgets


class QLogHandler(logging.Handler):
    def emit(self, record):
        LogMessageBox(record.levelno, record.msg, parent=QtWidgets.QApplication.focusWidget())


class LogMessageBox(QtWidgets.QMessageBox):
    LOGGING_LEVELS = {
        logging.INFO: QtWidgets.QMessageBox.Information,
        logging.WARNING: QtWidgets.QMessageBox.Warning,
        logging.ERROR: QtWidgets.QMessageBox.Warning,
        logging.CRITICAL: QtWidgets.QMessageBox.Critical
    }

    def __init__(self, level: int, message: str, parent=None):
        super().__init__(LogMessageBox.LOGGING_LEVELS[level], 'Specvizitor Message', message, parent=parent)
        self.show()


def qlog(func):
    @wraps(func)
    def wrapper(*args):
        root_logger = logging.getLogger()
        handler = QLogHandler()
        root_logger.addHandler(handler)

        res = func(*args)

        root_logger.removeHandler(handler)

        return res

    return wrapper



import logging
import pathlib
from dataclasses import dataclass
from platformdirs import user_config_dir, user_cache_dir

import pandas as pd
from astropy.table import Table

from .utils.params import LocalFile, Config, Cache
from .io import loader, writer


logger = logging.getLogger(__name__)


@dataclass
class RuntimeData:
    config_file: LocalFile = LocalFile(user_config_dir('specvizitor'), signature='Configuration file')
    cache_file: LocalFile = LocalFile(user_cache_dir('specvizitor'), signature='Cache')

    config: Config = None
    cache: Cache = None

    output_path: pathlib.Path = None
    cat: Table = None  # catalogue
    df: pd.DataFrame = None  # output data

    j: int = None  # index of the current object

    def __post_init__(self):
        self.config = Config.read(self.config_file)
        self.cache = Cache.read(self.cache_file)

    @property
    def id(self) -> int | None:
        try:
            return self.df.index[self.j]
        except (TypeError, IndexError):
            return

    @property
    def n_objects(self):
        return len(self.df)

    def create(self):
        df = pd.DataFrame(index=self.cat['id']).sort_index()
        df['comment'] = ''
        for i, cname in enumerate(self.config.review_form.checkboxes.keys()):
            df[cname] = False

        self.df = df

    def read(self):
        if self.output_path is not None:
            df = pd.read_csv(self.output_path, index_col='id')
            df['comment'] = df['comment'].fillna('')

            checkboxes = {key: value for key, value in self.config.review_form.checkboxes.items() if key in df.columns}
            for cname in df.columns:
                if pd.api.types.is_bool_dtype(df[cname]) and cname not in checkboxes.keys():
                    checkboxes[cname] = cname.capitalize()

            self.config.review_form.checkboxes = checkboxes
            self.config.save(self.config_file)

            self.df = df

            cat = loader.load_cat(self.config.loader.cat.filename,
                                  translate=self.config.loader.cat.translate)

            if cat is None:
                # init a catalogue with a single column of IDs
                cat = Table([self.df.index.values], names=('id',))
                cat.add_index('id')

            self.cat = cat

    def save(self):
        if self.output_path is not None:
            writer.save(self.df, self.output_path)
            logger.info('Project saved (path: {})'.format(self.output_path))



# import eazy

import pyqtgraph as pg
from pyqtgraph.Qt import QtCore, QtWidgets


class Eazy(QtWidgets.QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setEnabled(False)

        grid = QtWidgets.QGridLayout()

        self._run_eazy_button = QtWidgets.QPushButton("Run Eazy")
        self._run_eazy_button.clicked.connect(self.run_eazy)
        grid.addWidget(self._run_eazy_button, 1, 1)

        self.setLayout(grid)

    def run_eazy(self):
        pass
        # eazy_inst = eazy.photoz.PhotoZ(param_file=self._parent.config['data']['eazy']['param_file'],
        #                                translate_file=self._parent.config['data']['eazy']['translate_file'],
        #                                zeropoint_file=self._parent.config['data']['eazy']['zeropoint_file'],
        #                                load_prior=False, load_products=False, n_proc=-1)



from astropy.utils.decorators import lazyproperty

from pyqtgraph.Qt import QtWidgets

from .AbstractWidget import AbstractWidget

from ..runtime import RuntimeData
from ..utils import params
from ..io.loader import get_filename


class ViewerElement(AbstractWidget):
    def __init__(self, rd: RuntimeData, cfg: params.ViewerElement, parent=None):
        super().__init__(rd=rd, cfg=cfg, parent=parent)

    @lazyproperty
    def _filename(self):
        return get_filename(self.rd.config.loader.data.dir, self.cfg.search_mask, self.rd.id)

    def reset_view(self):
        pass

    def load_object(self):
        del self._filename
